class User < ApplicationRecord
  attr_accessor :password
  before_save :encrypt_password
  has_many :questionnaires

  validates :username, 
    {
      presence: true,
      uniqueness: true,
      length: { in: 4..18 }
    }
  validates :email,
    presence: true
  validates :password, confirmation: true, if: :password_required?

  def get_questionnaires_page(page)
    self.questionnaires.order(updated_at: :desc).paginate(page: page, per_page: 5)
  end

  def encrypt_password
    return if password.blank?
    if new_record?
      self.salt = Digest::SHA1.hexdigest(Time.now.to_s + self.username)
    end
    self.encrypted_password = User.encrypt(self.password, self.salt)
  end

  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest(salt+password)
  end

  def self.authenticate(username, password)
    user = User.find_by username: username
    user && user.authenticated?(password) ? user : nil
  end

  def authenticated?(password)
    self.encrypted_password == User.encrypt(password, self.salt)
  end

  def password_required?
    self.new_record? || !self.password.blank?
  end
end
